local Version = 1.3
local Development = false

RT_Main = {

	OnLoad = function()
		this:RegisterEvent("PLAYER_LOGIN")
		this:RegisterEvent("RAID_ROSTER_UPDATE")
		
		SLASH_RTFAKERAID1 = "/rtfake";
		SlashCmdList["RTFAKERAID"] = RT_Main.FakeRaid;
		
		StaticPopupDialogs["RT_SAVE_TEMPLATE"] = {
			text = "Save current tasks as template:",
			button1 = "Save",
			button2 = "Cancel",
			OnAccept = function()
					RT_Template.Save(getglobal(this:GetParent():GetName().."EditBox"):GetText())
				end,
			timeout = 0,
			whileDead = 1,
			hasEditBox = 1,
			hideOnEscape = 1
		}
		
		StaticPopupDialogs["RT_LOAD_TEMPLATE"] = {
			text = "Load task template (available: %s):",
			button1 = "Load",
			button2 = "Cancel",
			OnAccept = function()
					RT_Template.Load(getglobal(this:GetParent():GetName().."EditBox"):GetText())
				end,
			timeout = 0,
			whileDead = 1,
			hasEditBox = 1,
			hideOnEscape = 1
		}
		
		StaticPopupDialogs["RT_DELETE_TEMPLATE"] = {
			text = "Delete task template (available: %s):",
			button1 = "Delete",
			button2 = "Cancel",
			OnAccept = function()
					RT_Template.Delete(getglobal(this:GetParent():GetName().."EditBox"):GetText())
				end,
			timeout = 0,
			whileDead = 1,
			hasEditBox = 1,
			hideOnEscape = 1
		}
		
		StaticPopupDialogs["RT_REMOVE_TASK"] = {
			text = "Do you want to delete the following task '%s' ?",
			button1 = "Yes",
			button2 = "No",
			OnAccept = function(frame)
				RT_Task.Remove(frame);
			end,
			timeout = 0,
			whileDead = 1,
			hideOnEscape = 1
		};
		
	end,
	
	OnEvent = function()
		RT_Main.Debug("EVENT: " .. event)
		if(event == "PLAYER_LOGIN") then
			if(RT_Data == nil or Development == true or RT_Data.Version ~= Version) then
				RT_Data = {
					ClassFilter = {
						Druid = true, Priest = true, Shaman = true,
						Paladin = true,	Warrior = true, Warlock = true, 
						Mage = false, Rogue = false, Hunter = false
					},
					Version = Version,
					TaskColumns = 2,
					FrameLocked = false,
					MinimapPos = 45,
					Templates = {},
				}
			end
			RT_Main.ScanRaid()
			RT_Main.PositionMinimapButton()

			for class, toggle in pairs(RT_Data.ClassFilter) do
				getglobal("RaidTask_Options_"..class):SetChecked(toggle)
			end
		elseif(event == "RAID_ROSTER_UPDATE") then
			RT_Main.ScanRaid()
		end
	end,
	
	-- Creates a fake raid used for testing
	FakeRaid = function()
		for i = 1, 40 do
			local group = math.mod(i, 8)+1
			local name = "Player"..i
			if(i < 6) then
				class = "Druid"
			elseif(i < 11) then
				class = "Priest"
			elseif(i < 16) then
				class = "Paladin"
			elseif(i < 21) then
				class = "Warrior"
			elseif(i < 26) then
				class = "Mage"
			elseif(i < 31) then
				class = "Warlock"
			elseif(i < 36) then
				class = "Hunter"
			elseif(i < 41) then
				class = "Rogue"
			end
			
			RT_Player.Create(name, class, group, true)
		end
		RT_Main.RenderPlayerList()
	end,

	PositionMinimapButton = function()
		local pos = RT_Data.MinimapPos
		RaidTask_MinimapButton:SetPoint("TOPLEFT", "Minimap", "TOPLEFT", 55-(80*cos(pos)), (80*sin(pos))-55);
		RaidTask_Options_MinimapSliderText:SetText(pos .. "\194\176");
	end,
	
	Announce = function(chan)

		local tasks = RT_Task.Active
		
		if(table.getn(tasks) > 0 ) then
			SendChatMessage("## Raid Task ("..Version..") ##", chan)
		
			for _, task in ipairs(tasks) do
				local chatString = ""

				local assisters = RT_Task.GetAssisters(task)
				local targets = RT_Task.GetTargets(task)
				local title = RT_Task.GetTitle(task)
				
				if(table.getn(targets) > 0 or table.getn(assisters) > 0) then
					local assisterNames = {}
					local targetNames = {}
					
					for _, player in ipairs(assisters) do
						table.insert(assisterNames, player.Name)
					end
					
					for _, player in ipairs(targets) do
						table.insert(targetNames, player.Name)
					end
				
					if(string.len(title) > 0) then chatString = chatString .. "(" .. title .. ") " end
					chatString = chatString .. table.concat(targetNames, "+") .. " = " .. table.concat(assisterNames, ', ')
					SendChatMessage(chatString, chan)
				end
			end
		end
	end,
	
	ScanRaid = function()
		local raidCount = GetNumRaidMembers()
		if(raidCount > 0) then
			
			local active = RT_Player.Active
			local new = {}
			
			for _, player in pairs(active) do
				player.InRaid = false
			end
			
			for i = 1, raidCount do
				local name, _, group, _, class, _, _, online, _ = GetRaidRosterInfo(i)
				if(active[name] == nil)then
					table.insert(new,{name, class, group, online})
				else
					active[name].InRaid = true
					RT_Player.SetOnline(active[name], online)
				end
			end
			
			for name, frame in pairs(active) do
				if(frame.InRaid == false) then RT_Player.Remove(frame) end
			end
			
			for _, player in pairs(new) do
				RT_Player.Create(unpack(player))
			end
			
			if(table.getn(new) > 0) then RT_Main.RenderPlayerList() end
			
		end
	end,
	
	CTRAMTs = function()
		if(CT_RATarget ~= nil and CT_RATarget.MainTanks ~= nil and table.getn(CT_RATarget.MainTanks) > 0) then
			for index, tank in pairs(CT_RATarget.MainTanks) do
				local task = nil
				if(index == 1) then
					task = RT_Task.Create("MT")
				else
					task = RT_Task.Create("OT" .. index-1)
				end
				local targetsFrame = RT_Task.GetTargetsFrame(task)
				RT_Player.Active[tank[2]]:SetParent(targetsFrame)
				RT_Task.Render(targetsFrame)
			end
			RT_Main.RenderPlayerList()
			RT_Main.PositionTasks()
		end
	end,
	
	Hide = function(frame)
		frame:ClearAllPoints()
		frame:SetParent(UIParent)
		frame:Hide()
	end,
	
	Render = function(frame)
		if(frame ~= UIParent) then
			if(frame == RaidTask_PlayerList) then
				RT_Main.RenderPlayerList()
			else
				RT_Task.Render(frame)
			end
		end
	end,
	
	ForceRender = function()
		RT_Main.RenderPlayerList()
		for _, frame in ipairs(RT_Task.Active) do
			RT_Task.SubRender(frame)
		end
	end,
	
	RenderPlayerList = function()
		local playerCount = RT_Player.ListChildren(RaidTask_PlayerList, 0, -15)
		local height = playerCount*11+24
		if(height < 40) then height = 40 end
		if(RaidTask_PlayerList.Minimized ~= true) then
			RaidTask_PlayerList_Wrapper:SetHeight(height)
		end
	end,
	
	PositionTasks = function()
		local tasks = RT_Task.Active
		local colCount = RT_Data.TaskColumns
		for index, task in ipairs(tasks) do
			RT_Main.Debug("POSITIONING Task Frame:" .. index)
			task:ClearAllPoints()
			
			if(math.mod(index, colCount) == 1) then
				if(index == 1) then
					task:SetPoint("TOPLEFT", RaidTask_Main, "BOTTOMLEFT")
				else
					task:SetPoint("TOP", tasks[index-colCount], "BOTTOM")
				end
			elseif(index > colCount) then
				task:SetPoint("TOP", tasks[index-colCount], "BOTTOM")
			else
				task:SetPoint("TOPLEFT", tasks[index-1], "TOPRIGHT")
			end
			
		end
	end,
	
	-- Detects collision between two frames, also requires both frames to be visible
	-- so we don't have things snapping to hidden frames.
	FramesOverlap = function(frameOne, frameTwo)
		local scaleOne, scaleTwo = frameOne:GetEffectiveScale(), frameTwo:GetEffectiveScale();
		--RT_Main.Debug(frameOne:GetName() .. " <collision detect> " .. frameTwo:GetName())
		return ((frameOne:GetLeft()*scaleOne) < (frameTwo:GetRight() *scaleTwo)) and ((frameTwo:GetLeft()*scaleTwo) < (frameOne:GetRight()*scaleOne)) and 
			   ((frameOne:GetBottom()*scaleOne) < (frameTwo:GetTop()*scaleTwo)) and ((frameTwo:GetBottom()*scaleTwo) < (frameOne:GetTop()*scaleOne)) and 
			   frameTwo:IsVisible() and frameOne:IsVisible();
	end,

	Debug = function(msg)
		-- Prints a nice debug message in a redish color if Development is set to true
		if(Development == true) then
			DEFAULT_CHAT_FRAME:AddMessage("RaidTask Debug: " .. msg, 201/255, 151/255, 151/255)
		end
	end,
	
}