RT_Player = {

	Active = {},
	Inactive = {},
	Counter = 1,
	
	Colors = {
		Druid	= {1, 0.49, 0.039},
		Hunter	= {0.67, 0.831, 0.451},
		Mage	= {0.411, 0.8, 0.941},
		Rogue	= {1, 0.96, 0.411},
		Warlock = {0.58, 0.509, 0.788},
		Warrior = {0.78, 0.611, 0.431},
		Paladin	= {0.96, 0.549, 0.725},
		Shaman	= {0.96, 0.549, 0.725},
		Priest	= {1, 1, 1},
		Offline = {0.5, 0.5, 0.5}
	},
	
	Remove = function(frame)
		local oldParent = frame:GetParent()
		local active = RT_Player.Active
		local name = frame.Name
		
		if(active[name] ~= nil) then
			RT_Main.Debug("REMOVING: " .. name)
			table.insert(RT_Player.Inactive, frame)
			frame.X, frame.Y = 0, 0
			RT_Main.Hide(frame)
			active[name] = nil
		end
		
		RT_Main.Render(oldParent)
	end,
	
	Create = function(name, class, group, online)
		local active, inactive = RT_Player.Active, RT_Player.Inactive
		local frame = nil
		
		if(inactive[1] ~= nil) then
			RT_Main.Debug("REUSING Player Frame")
			frame = inactive[1]
			table.remove(inactive, 1)
			frame:SetParent(RaidTask_PlayerList)
		else
			frame = CreateFrame("Frame", "RT_Player_"..RT_Player.Counter, RaidTask_PlayerList, "RaidTask_PlayerFrame")
			RT_Player.Counter = RT_Player.Counter+1
		end
		
		frame.Class = class
		frame.Name = name
		frame.Group = group
		
		RT_Player.SetOnline(frame, online)
		RT_Player.SetText(frame, name)
		RT_Player.SetColor(frame)
		
		active[name] = frame
		frame:Show()
		
		return frame
	end,
	
	SetText = function(frame, name)
		RT_Player.GetTextFrame(frame):SetText(name)
	end,
	
	GetText = function(frame)
		return RT_Player.GetTextFrame(frame):GetText()
	end,
	
	SetColor = function(frame)
		if(frame.Online == true) then
			RT_Player.GetTextFrame(frame):SetTextColor(unpack(RT_Player.Colors[frame.Class]))
		else
			RT_Player.GetTextFrame(frame):SetTextColor(unpack(RT_Player.Colors.Offline))
		end
	end,
	
	SetOnline = function(frame, online)
		if(online == nil) then frame.Online = false
		else frame.Online = true end
		RT_Player.SetColor(frame)
	end,
	
	GetTextFrame = function(frame)
		return getglobal(frame:GetName() .. "_Text")
	end,
	
	ListChildren = function(parentFrame, xOffset, yOffset)
		local playerFrames = { parentFrame:GetChildren() }
		local position = 0
		
		for _, frame in ipairs(playerFrames) do
			-- Only position frames if thier class is allowed
			if(RT_Data.ClassFilter[frame.Class] == true) then
				frame.X, frame.Y = xOffset, yOffset+(-position*11)
				RT_Player.AttachToParent(frame)
				position = position+1
			else
				frame:Hide()
			end
		end
		return position
	end,
	
	Snap = function(frame)
		RT_Main.Debug("SNAPPING " .. frame:GetName())
		
		-- If it overlaps with the standard player list
		if(RT_Main.FramesOverlap(frame, RaidTask_PlayerList)) then
			RT_Player.SetParent(frame, RaidTask_PlayerList)
			return
		else
			-- Check to see if it overlaps with a task
			local tasks = RT_Task.Active
			for _, task in ipairs(tasks) do

				local assistersFrame = RT_Task.GetAssistersFrame(task)
				local targetsFrame = RT_Task.GetTargetsFrame(task)
				
				if(RT_Main.FramesOverlap(frame, assistersFrame)) then
					RT_Player.SetParent(frame, assistersFrame)
					return
				elseif(RT_Main.FramesOverlap(frame, targetsFrame)) then
					RT_Player.SetParent(frame, targetsFrame)
					return
				end
			end
		end
		
		-- It only gets here if we moved it to something it couldn't snap to
		-- so we need to snap it back to the original parent
		RT_Player.AttachToParent(frame)
	end,
	
	AttachToParent = function(frame)
		frame:ClearAllPoints()
		frame:SetPoint("TOP", frame:GetParent(), "TOP", frame.X, frame.Y)
		frame:Show()
	end,
	
	SetParent = function(frame, newParent)
		local oldParent = frame:GetParent()
		if(oldParent ~= newParent) then
			frame:SetParent(newParent)
			RT_Main.Render(newParent)
			RT_Main.Render(oldParent)
		else
			RT_Player.AttachToParent(frame)
		end
	end,
	
}