RT_Task = {
	
	Active = {},
	Inactive = {},
	Counter = 1,
	
	Create = function(title)
		local active, inactive = RT_Task.Active, RT_Task.Inactive
		local frame = nil
		
		if(inactive[1] ~= nil) then
			RT_Main.Debug("REUSING Task Frame")
			frame = inactive[1]
			frame:SetParent(RaidTask_Main)
			RT_Task.Maximize(frame)
			table.remove(inactive, 1)
		else
			RT_Main.Debug("CREATING Task Frame")
			frame = CreateFrame("Frame", "RaidTask_Task" .. RT_Task.Counter , RaidTask_Main, "RaidTask_TaskFrame")
			RT_Task.Counter = RT_Task.Counter+1
		end
		
		RT_Task.SetTitle(frame, title)
		table.insert(active, frame)
		frame:Show()
		return frame
	end,

	Remove = function(frame)				
		local active = RT_Task.Active
		
		for index, matchFrame in ipairs(active) do
			if(matchFrame == frame) then
				RT_Task.ResetFrame(frame, index)				
				RT_Main.PositionTasks()
				break
			end
		end
	end,
	
	ResetFrame = function(frame, index)
		local active = RT_Task.Active
		local inactive = RT_Task.Inactive
	
		-- Remove Targets
		local targets = RT_Task.GetTargets(frame)
		for _, player in ipairs(targets) do
			player:SetParent(RaidTask_PlayerList)
		end				
		
		-- Remove Assisters
		local assisters = RT_Task.GetAssisters(frame)
		for _, player in ipairs(assisters) do
			player:SetParent(RaidTask_PlayerList)
		end
	
		RT_Task.SubRender(frame)
		RT_Main.Render(RaidTask_PlayerList)
		RT_Main.Hide(frame)
		
		table.remove(active, index)
		table.insert(inactive, frame)
	end,
	
	RemoveAll = function()
		local active = RT_Task.Active
		local old = {}
		
		for _, task in ipairs(active) do table.insert(old, task) end
		for index, task in ipairs(old) do RT_Task.ResetFrame(task, index) end
	end,

	Minimize = function(frame)
		RT_Task.GetAssistersFrame(frame):Hide()
		RT_Task.GetTargetsFrame(frame):Hide()
		frame:SetHeight(18)
	end,
	
	Maximize = function(frame)
		RT_Task.GetAssistersFrame(frame):Show()
		RT_Task.GetTargetsFrame(frame):Show()
		RT_Task.SubRender(frame)
	end,
	
	Render = function(frame)
		-- "frame" here is actually the targets/assisters frame and not the task frame itself
		local playerCount = RT_Player.ListChildren(frame, 0, 0)
		frame:SetHeight(11+playerCount*11)
		RT_Task.SetHeight(frame:GetParent())
	end,
	
	SubRender = function(frame)
		RT_Task.Render(RT_Task.GetTargetsFrame(frame))
		RT_Task.Render(RT_Task.GetAssistersFrame(frame))
	end,
	
	SetHeight = function(frame)
		local targetsHeight = RT_Task.GetTargetsFrame(frame):GetHeight()
		local assistersHeight = RT_Task.GetAssistersFrame(frame):GetHeight()
		frame:SetHeight(targetsHeight+assistersHeight+32)
	end,
	
	GetTitle = function(frame)
		return RT_Task.GetTitleFrame(frame):GetText()
	end,
	
	SetTitle = function(frame, title)
		if(title == nil) then title = "" end
		return RT_Task.GetTitleFrame(frame):SetText(title)
	end,
	
	GetTitleFrame = function(frame)
		return getglobal(frame:GetName() .. "_Title")
	end,
	
	GetTargets = function(frame)
		return { RT_Task.GetTargetsFrame(frame):GetChildren() }
	end,
	
	GetTargetsFrame = function(frame)
		return getglobal(frame:GetName() .. "_Targets")
	end,
	
	GetAssisters = function(frame)
		return { RT_Task.GetAssistersFrame(frame):GetChildren() }
	end,
	
	GetAssistersFrame = function(frame)
		return getglobal(frame:GetName() .. "_Assisters")
	end,

}
	